///////////////////////////
// WINDOWS SPECIFIC PART //

#include "stdafx.h"

///////////////////////////

#include "res.h"
#include "datastructs.h"
#include "stuff.h"
#include "state.h"
#include "referee.h"
#include "player.h"
#include "manager.h"

CManager::CManager ()
{
	referee = NULL;
	for (char i = 0; i < PLAYER_COUNT; i++) {
		players_table.objects[i] = NULL;
		players_table.types[i] = HUMAN;
		players_table.iqs[i] = 2;
	}
}

CManager::~CManager ()
{
	for (char i = 0; i < PLAYER_COUNT; i++) delete players_table.objects[i];
}

bool CManager::New_Game ()
{
	if (!Reset_Players ()) return false;
	referee->Create_New_Game ();
	return true;
}

bool CManager::Reset_Players ()
{
	char i;
	for (i = 0; i < PLAYER_COUNT; i++) {
		delete players_table.objects[i]; players_table.objects[i] = NULL;
	}
	for (i = 0; i < PLAYER_COUNT; i++) {
		players_table.objects[i] = new CPlayer (referee, TColor (i), players_table.types[i], players_table.iqs[i]);
		if (players_table.objects[i] == NULL) return false;
	}
	return true;
}

bool CManager::Execute_Move (TColor p, CMove* m)
{
	if (!referee->Check_Move (m, p)) return false;
	if (!referee->Execute_Move (m, NULL)) return false;
	referee->Clear_Undo_List();
	Change_Player_On_Turn (p);
	bool val;
	if (!referee->Get_Game_Ended (val)) return false;
	return true;
}

void CManager::Change_Player_On_Turn (TColor last_col)
{
	if (last_col == WHITE) referee->Set_Player_On_Turn (BLACK);
	else if (last_col == BLACK) referee->Set_Player_On_Turn (WHITE);
}

bool CManager::Set_Player (TColor c, TPlayer_Type t, int iq)
{
	players_table.types[c] = t;
	players_table.iqs[c] = iq;
	if (!Reset_Players ()) return false;
	return true;
}

bool CManager::Get_Move (CMove* move, TColor c)
{
	if (c == NO_COLOR) return false;
	else if (players_table.objects[c] == NULL) return false;
	bool val;
	if (!referee->Get_Game_Ended (val)) return false;
	if (!val) if (players_table.objects[c]->Get_Move (move)) return true;
	return false;
}

bool CManager::Force_Move (CMove* move, TColor c)
{
	if (c == NO_COLOR) return false;
	else if (players_table.objects[c] == NULL) return false;
	bool val;
	if (!referee->Get_Game_Ended (val)) return false;
	if (!val) if (players_table.objects[c]->Force_Move (move)) return true;
	return false;
}

void CManager::DataAsText(char* text, int who)
{
	if (who == 1) { referee->DataAsText(text); return; }
	if ((who == 2) || (who == 3)) { 
		if (players_table.objects[who - 2] != NULL)
			players_table.objects[who - 2]->DataAsText(text);
		else strcat(text, "Object does not exist.");
		return;
	}

	strcat (text, "Manager:##");
	strcat(text, "Referee: "); if (referee != NULL) strcat(text, "OK#"); else strcat(text, "NULL#");
	strcat(text, "Players table: color object type iq-level#");
	strcat(text, "WHITE ");
	if (players_table.objects[0] != NULL) strcat(text, "OK "); else strcat(text, "NULL ");
	if (players_table.types[0] == HUMAN) strcat(text, "HUMAN "); else strcat(text, "COMPUTER ");
	char letter[2]; letter[1] = 0;
	*letter = players_table.iqs[0] + '0'; strcat(text, letter); strcat(text, "#");
	strcat(text, "BLACK ");
	if (players_table.objects[1] != NULL) strcat(text, "OK "); else strcat(text, "NULL ");
	if (players_table.types[1] == HUMAN) strcat(text, "HUMAN "); else strcat(text, "COMPUTER ");
	*letter = players_table.iqs[1] + '0'; strcat(text, letter); strcat(text, "#");
}
