///////////////////////////
// WINDOWS SPECIFIC PART //

#include "stdafx.h"

///////////////////////////

#include "res.h"
#include "datastructs.h"
#include "stuff.h"
#include "state.h"
#include "referee.h"
#include "player.h"

#include <time.h>

CPlayer::CPlayer (CReferee* ref, TColor c, TPlayer_Type t, int iq)
{ 
	referee = ref;
	type = t;
	color = c;
	tree_root = NULL;
	tree_depth = iq;
	can_force_move = forcing_move = move_forced = false;
	srand((unsigned)time(NULL));
}

bool CPlayer::Create_Tree (m_CTree_Node<CState*>* node, TColor col, int depth)
{
	if (move_forced) return true;
	m_CList<CState*> state_list;
	if (!referee->Get_All_States (col, &state_list, node->Get_Value ())) return false;
	if (state_list.Empty ()) return true;
	m_CList<m_CTree_Node<CState*>*>* son_list = node->Get_List_Of_Sons ();
	m_CTree_Node<CState*>* new_son = NULL;
	for (int i = 0; i != state_list.Number_Of_Items (); i++) {
		new_son = NULL;
		new_son = new m_CTree_Node<CState*> (state_list.Get_Value (i));
		if (new_son == NULL) return false;
		if (!new_son->Create ()) return false;
		if (!son_list->Insert_Value (new_son)) return false;
		can_force_move = true;
		if (depth < tree_depth) {
			if (col == WHITE) {
				if (!Create_Tree (new_son, BLACK, depth + 1)) return false;
			}
			else {
				if (!Create_Tree (new_son, WHITE, depth + 1)) return false;
			}
			m_CList<m_CTree_Node<CState*>*>* new_son_list = new_son->Get_List_Of_Sons ();
			int max_val = 0, val;
			bool max;
			if (((color == WHITE) && ((depth % 2) == 0)) || ((color == BLACK) && ((depth % 2) == 1)))
				max = true;
			else max = false;
			if (!new_son_list->Empty()) 
				max_val = new_son_list->Get_Value (0)->Get_Value ()->Get_Eval_Value ();
			for (int j = 1; j < new_son_list->Number_Of_Items (); j++) {
				val = new_son_list->Get_Value (j)->Get_Value ()->Get_Eval_Value ();
				if ((max && (val > max_val)) || (!max && (val < max_val))) max_val = val;
			}
			new_son->Get_Value ()->Set_Eval_Value (max_val);
			for (int k = 0; k != new_son_list->Number_Of_Items (); k++) {
				delete new_son_list->Get_Value(k)->Get_Value();
				delete new_son_list->Get_Value(k);
			}
			new_son_list->Clear();
		}
		else {
			int j;
			m_CList<CMove*> mv_list;
			if (!referee->Get_All_Moves (WHITE, &mv_list, new_son->Get_Value ())) return false;
			int w = mv_list.Number_Of_Items ();
			for (j = 0; j != mv_list.Number_Of_Items (); j++) delete mv_list.Get_Value (j);
			mv_list.Clear ();
			if (!referee->Get_All_Moves (BLACK, &mv_list, new_son->Get_Value ())) return false;
			int b = mv_list.Number_Of_Items ();
			for (j = 0; j != mv_list.Number_Of_Items (); j++) delete mv_list.Get_Value (j);
			new_son->Get_Value ()->Evaluate (w, b); // pass numbers of moves and evaluate
		}
	}
	return true;
}

bool CPlayer::Get_First_Node_Of_Way_To_Best_State (m_CTree_Node<CState*>*& node)
{
	m_CList<m_CTree_Node<CState*>*>* son_list = tree_root->Get_List_Of_Sons ();
	int index = (int)(rand () * (son_list->Number_Of_Items () / (float)RAND_MAX)); // random move
	if (tree_depth == 0) { node = son_list->Get_Value (index); return true; }
	int max_val, val;
	max_val = son_list->Get_Value (index)->Get_Value ()->Get_Eval_Value ();
	for (int j = 0; j != son_list->Number_Of_Items (); j++) {
		val = son_list->Get_Value (j)->Get_Value ()->Get_Eval_Value ();
		if (((color == WHITE) && (val > max_val)) || ((color == BLACK) && (val < max_val))) { 
			max_val = val; index = j;
		}
	}
	node = son_list->Get_Value (index); // node of the best state
	return true;
}

void CPlayer::Destroy_Tree (m_CTree_Node<CState*>* node)
{
	m_CList<m_CTree_Node<CState*>*>* son_list = node->Get_List_Of_Sons ();
	if (!son_list->Empty ())
		for (int i = 0; i != son_list->Number_Of_Items (); i++)
			Destroy_Tree (son_list->Get_Value (i));
	delete node->Get_Value ();
	delete node;
}

bool CPlayer::Get_Move (CMove* move)
{
	move_forced = false;
	tree_root = NULL;
	tree_root = new m_CTree_Node<CState*> (NULL);
	if (tree_root == NULL) return false;
	if (!tree_root->Create ()) return false;
	if (!Create_Tree (tree_root, color, 1)) { can_force_move = false; return false; }
	can_force_move = false;
	while (forcing_move) ;
	if (!move_forced) {
		if (tree_root->Get_List_Of_Sons ()->Empty ()) return false; // no move
		m_CTree_Node<CState*>* my_node = NULL;
		if (!Get_First_Node_Of_Way_To_Best_State (my_node)) return false;
		*move = *my_node->Get_Value ()->Get_Last_Move (); // best move
	}
	Destroy_Tree (tree_root); // delete the whole tree (including root)
	return true;
}

bool CPlayer::Force_Move (CMove* move)
{
	if (!can_force_move) return false;
	forcing_move = true;
	if (tree_root->Get_List_Of_Sons ()->Empty ()) { forcing_move = false; return false; } // no move
	m_CTree_Node<CState*>* my_node = NULL;
	if (!Get_First_Node_Of_Way_To_Best_State (my_node)) { forcing_move = false; return false; }
	*move = *my_node->Get_Value ()->Get_Last_Move (); // best move
	forcing_move = false;
	move_forced = true;
	return true;
}

void CPlayer::DataAsText(char* text)
{
	strcat (text, "Player:##");
	strcat(text, "Referee: "); if (referee != NULL) strcat(text, "OK#"); else strcat(text, "NULL#");
	strcat(text, "Color: ");
	if (color == WHITE) strcat(text, "WHITE#"); else strcat(text, "BLACK#");
	strcat(text, "Type: ");
	if (type == HUMAN) strcat(text, "HUMAN#"); else strcat(text, "COMPUTER#");
	strcat(text, "Tree depth: "); 
	char letter[2]; letter[1] = 0;
	*letter = tree_depth + '0'; strcat(text, letter); strcat (text, "#");
}
