///////////////////////////
// WINDOWS SPECIFIC PART //

#include "stdafx.h"

///////////////////////////

#include "res.h"
#include "datastructs.h"
#include "stuff.h"
#include "state.h"
#include "referee.h"

CReferee::CReferee ()
{
	state = NULL;
	must_jump = true;
	nojump_move_count = 0;
	last_move_mistake = 0;
}

CReferee::~CReferee ()
{
	for (int k = 0; k != move_list.Number_Of_Items (); k++) delete move_list.Get_Value (k);
	for (int j = 0; j < undo_list.Number_Of_Items (); j++) delete undo_list.Get_Value (j);
}

void CReferee::Create_New_Game ()
{
	state->Reset ();
	state->Set_Player_On_Turn (PLAYER_ON_FIRST_TURN);
	state->Set_Victory (false);
	state->Set_Game_Ended (false);
	nojump_move_count = 0;
	last_move_mistake_positions.Clear ();
}

enum TDirection { Right_Up, Left_Up, Left_Down, Right_Down, No_Dir };

bool CReferee::Get_All_Moves_From_Position (CPosition& pos, m_CList<CMove*>* list, CState* st)
{
	if (st == NULL) st = state;
	CBoard* board = st->Get_Board ();
	if (board->Get_Figure (pos) == NULL) return true;
	TColor color = board->Get_Figure (pos)->Get_Color ();
	TFigure_Type type = board->Get_Figure (pos)->Get_Type ();
	const char DIR_COUNT = 4; // four directions of move (diagonal)
	char add_v, add_h;
	CPosition newpos;
	for (char i = 0; i < DIR_COUNT; i++) {
		if (i == Right_Up) { add_v = 1; add_h = 1; }
		if (i == Left_Up) { add_v = 1; add_h = -1; }
		if (i == Left_Down) { add_v = -1; add_h = -1; }
		if (i == Right_Down) { add_v = -1; add_h = 1; }
		if (type == MAN) // man cannot go back
			if ((color == WHITE) && (add_v == -1)) continue;
			else if ((color == BLACK) && (add_v == 1)) continue;
		if ((pos.Get_Vertical_Index () + add_v < 0) ||
			(pos.Get_Vertical_Index () + add_v >= FIELD_COUNT_VERTICAL) ||
			(pos.Get_Horizontal_Index () + add_h < 0) ||
			(pos.Get_Horizontal_Index () + add_h >= FIELD_COUNT_HORIZONTAL))
			continue; // we cannot go out of the game board
		newpos = CPosition (pos.Get_Vertical_Index () + add_v, pos.Get_Horizontal_Index () + add_h);
		if (board->Get_Figure (newpos) == NULL) { // empty position ? yes -> insert move in the list
			CMove* move = NULL;
			move = new CMove;
			if (move == NULL) return false;
			if (!move->Insert_Position (pos)) return false;
			if (!move->Insert_Position (newpos)) return false;
			move->Set_Player(color);
			if (!list->Insert_Value (move)) return false;
		}
	}
	CMove move;
	if (!move.Insert_Position (pos)) return false;
	move.Set_Player(color);
	if (!Get_All_Jumps_From_Position (list, &move, pos, No_Dir, st, color, type)) return false;
	return true;
}

bool CReferee::Get_All_Jumps_From_Position (m_CList<CMove*>* list, CMove* move, CPosition& pos, char from, CState* st, TColor color, TFigure_Type type)
{
	CBoard* board = st->Get_Board ();
	const char DIR_COUNT = 4; // four directions of jump (diagonal)
	char add_v, add_h;
	CPosition newpos;
	if ((from == Left_Down) || (from == Right_Down)) from -= 2;
	else if ((from == Right_Up) || (from == Left_Up)) from += 2; // turn round direction ! if not done, we would jump back
	for (char i = 0; i < DIR_COUNT; i++) {
		if (i == Right_Up) { add_v = 2; add_h = 2; }
		if (i == Left_Up) { add_v = 2; add_h = -2; }
		if (i == Left_Down) { add_v = -2; add_h = -2; }
		if (i == Right_Down) { add_v = -2; add_h = 2; }
		if (i == from) continue; // don't jump back !
		if (type == MAN) // man cannot jump back
			if ((color == WHITE) && (add_v == -2)) continue;
			else if ((color == BLACK) && (add_v == 2)) continue;
		if ((pos.Get_Vertical_Index () + add_v < 0) ||
			(pos.Get_Vertical_Index () + add_v >= FIELD_COUNT_VERTICAL) ||
			(pos.Get_Horizontal_Index () + add_h < 0) ||
			(pos.Get_Horizontal_Index () + add_h >= FIELD_COUNT_HORIZONTAL))
			continue; // we cannot jump out of the game board
		newpos = CPosition (pos.Get_Vertical_Index () + add_v / 2, pos.Get_Horizontal_Index () + add_h / 2);
		if (board->Get_Figure (newpos) == NULL) continue;
		else if (board->Get_Figure (newpos)->Get_Color () == color) continue; // we must jump over other color man than our jumping one is
		CState st_new; // we need new state to be able to move to positions which are not empty yet, but can be after some jumps, or to not overjump some figure twice, etc.
		if (!st_new.Create (false)) return false;
		st_new = *st;
		st_new.Remove_Figure (newpos);
		newpos = CPosition (pos.Get_Vertical_Index () + add_v, pos.Get_Horizontal_Index () + add_h);
		CFigure* fig = st_new.Remove_Figure (pos);
		if (board->Get_Figure (newpos) == NULL) { // empty position ? yes -> insert move in the list
			if (type != KING) if (Change_To_King (color, newpos)) type = KING; // last row -> change to king ?
			st_new.Put_Figure (newpos, fig);		
			CMove* newmove = NULL;
			newmove = new CMove;
			if (newmove == NULL) return false;
			*newmove = *move;
			if (!newmove->Insert_Position (newpos)) return false;
			if (!list->Insert_Value (newmove)) return false;
			if (!Get_All_Jumps_From_Position (list, newmove, newpos, i, &st_new, color, type)) // next level of jump
				return false;
		}
	}
	return true;
}

bool CReferee::Check_Move (CMove* move, TColor c)
{
	for (int k = 0; k != move_list.Number_Of_Items (); k++) delete move_list.Get_Value (k);
	move_list.Clear ();
	if (move->Get_Player() != c) return false;
	if (!Get_All_Moves (c, &move_list, state)) return false;
	CMove* m;
	int j;
	for (int i = 0; i != move_list.Number_Of_Items (); i++) {
		m = move_list.Get_Value (i);
		if (move->Number_Of_Positions () > m->Number_Of_Positions ()) continue; // our move is longer
		for (j = 0; j != move->Number_Of_Positions (); j++)
			if (!(m->Get_Position (j) == move->Get_Position (j))) break;
		if (j == move->Number_Of_Positions ()) return true; // all positions of our move are same as ones of some move in move_list
	}
	return false;
}

void CReferee::Move_Filter (m_CList<CMove*>* list)
{
	bool jump_in = false;
	int i = 0;
	while (i != list->Number_Of_Items ())
		if (!list->Get_Value (i)->Is_Jump ()) i++; else break;
	if (i != list->Number_Of_Items ()) jump_in = true;
	i = 0;
	CMove* move;
	while (i != list->Number_Of_Items ()) {
		move = list->Get_Value (i);
		if (!move->Is_Jump ()) 
			if (jump_in) { delete list->Get_Value(i); list->Remove_Item (i); }
			else i++;
		else {
			if (move->Number_Of_Positions () == 2) { i++; continue; } // simple jump is always allowed
			bool next = true;
			CMove* m;
			for (int j = 0; j != list->Number_Of_Items (); j++) {
				m = list->Get_Value (j);
				if ((m->Is_Jump ()) && (m->Number_Of_Positions () > 2))
					for (int k = 2; k != move->Number_Of_Positions (); k++) {
						if (k == m->Number_Of_Positions ()) break;
						if (move->Get_Position (k) == m->Get_Position (k)) {
							if (((k + 1) == move->Number_Of_Positions ()) &&
								((k + 1) < m->Number_Of_Positions ())) {
								delete list->Get_Value(i); list->Remove_Item (i); 
								next = false;
								break;
							}
						}
						else break;
					}
				if (!next) break;	
			}
			if (next) i++;
		}
	}
}

bool CReferee::Set_Move_Mistakes (CMove* move)
{
	last_move_mistake = 0;
	last_move_mistake_positions.Clear ();
	if (!move->Is_Jump ()) { // our move is not jump
		CMove* m;
		for (int i = 0; i != move_list.Number_Of_Items (); i++) {
			m = move_list.Get_Value (i);
			if (m->Is_Jump ()) { // (missed) jump in the list
				last_move_mistake = 1;
				if (m->Get_Position (0) == move->Get_Position (0)) { // we simply move with the figure we should jump
					if (!last_move_mistake_positions.Insert_Value (move->Get_Position (1)))
						return false;
				}
				else 
					if (!last_move_mistake_positions.Insert_Value (m->Get_Position (0))) // jump with other figure
						return false;
			} 
		}
	}
	else {
		if (move->Number_Of_Positions () == 2) return true; // simple jump is always allowed
		CMove* m;
		for (int i = 0; i != move_list.Number_Of_Items (); i++) {
			m = move_list.Get_Value (i);
			if ((m->Is_Jump ()) && (m->Number_Of_Positions () > 2))
				for (int k = 2; k != move->Number_Of_Positions (); k++) {
					if (k == m->Number_Of_Positions ()) break;
					if (move->Get_Position (k) == m->Get_Position (k)) {
						if (((k + 1) == move->Number_Of_Positions ()) &&
							((k + 1) < m->Number_Of_Positions ())) {
							last_move_mistake = 2;
							if (!last_move_mistake_positions.Insert_Value (move->Get_Position (move->Number_Of_Positions () - 1)))
								return false;
							return true;
						}
					}
					else break;
				}
		}
	}
	return true;
}

bool CReferee::Execute_Move (CMove* move, CState* st)
{
	if (st == NULL) st = state;
	if (st->Game_Ended()) return false;
	CFigure* fig;
	if (!move->Is_Jump ()) { // simple move
		fig = st->Remove_Figure (move->Get_Position (0));
		CPosition pos = move->Get_Position (1);
		st->Put_Figure (pos, fig);
		if (fig->Get_Type () != KING) if (Change_To_King (fig->Get_Color(), pos)) st->Change_To_King (pos);
		if (st == state) nojump_move_count++;
	}
	else { // (multi-)jump
		CPosition pos1, pos2;
		for (int i = 0; i != move->Number_Of_Positions () - 1; i++) {
			pos1 = move->Get_Position (i);
			pos2 = move->Get_Position (i + 1);
			fig = st->Remove_Figure (pos1);
			st->Put_Figure (pos2, fig);
			if (fig->Get_Type () != KING) if (Change_To_King (fig->Get_Color(), pos2)) st->Change_To_King (pos2);
			pos1 = CPosition ((pos1.Get_Vertical_Index () + pos2.Get_Vertical_Index ()) / 2,
				(pos1.Get_Horizontal_Index () + pos2.Get_Horizontal_Index ()) / 2);
			st->Remove_Figure (pos1); // remove overjumped figure
		}
		if (st == state) nojump_move_count = 0;
	}
	if (!must_jump) if (!Set_Move_Mistakes (move)) return false;
	if (!st->Set_Last_Move (move)) return false; // save last move
	for (int i = 0; i != move_list.Number_Of_Items (); i++) delete move_list.Get_Value (i);
	move_list.Clear ();
	return true;
}

bool CReferee::Change_To_King (TColor c, CPosition& pos)
{
	CBoard* board = state->Get_Board ();
	if (((c == WHITE) && (pos.Get_Vertical_Index () == FIELD_COUNT_VERTICAL - 1)) ||
		((c == BLACK) && (pos.Get_Vertical_Index () == 0))) // a man at last row ?
		return true;
	return false;
}

bool CReferee::Get_All_Moves (TColor color, m_CList<CMove*>* list, CState* st)
{
	if (st == NULL) st = state;
	CBoard* board = st->Get_Board ();
	CPosition pos;
	for (char i = 0; i < FIELD_COUNT_VERTICAL; i++) {
		pos.Set_Vertical_Index (i);
		for (char j = 0; j < FIELD_COUNT_HORIZONTAL; j++) {
			pos.Set_Horizontal_Index (j);
			if (board->Get_Figure (pos) != NULL)
				if (board->Get_Figure (pos)->Get_Color () == color)
					if (!Get_All_Moves_From_Position (pos, list, st)) return false;
		}
	}
	if (must_jump) Move_Filter (list);
	return true;
}

bool CReferee::Get_All_States (TColor color, m_CList<CState*>* list, CState* st)
{
	if (st == NULL) st = state;
	m_CList<CMove*> mv_list;
	if (!Get_All_Moves (color, &mv_list, st)) return false;
	CMove* m, m_passed;
	CState* st_new = NULL;
	for (int i = 0; i != mv_list.Number_Of_Items (); i++) {
		m = mv_list.Get_Value (i);
		m_passed = *m;
		for (char j = m->Number_Of_Positions (); j >= 2;) {
			st_new = NULL;
			st_new = new CState;
			if (st_new == NULL) return false;
			if (!st_new->Create (false)) return false;
			*st_new = *st;
			if (!Execute_Move (&m_passed, st_new)) return false;
			if (!list->Insert_Value (st_new)) return false;
			m_passed.Remove_Position (j - 1); j--;
		}
	}
	for (int k = 0; k != mv_list.Number_Of_Items (); k++) delete mv_list.Get_Value (k);
	return true;
}

bool CReferee::Get_Game_Ended (bool& val)
{
	if (state->Game_Ended ()) { val = true; return true; }
	if (nojump_move_count >= MAX_NOJUMP_MOVE_COUNT) {
		state->Set_Game_Ended (true);
		val = true;
		return true;
	}
	m_CList<CMove*> mv_list;
	TColor col = state->Whos_On_Turn ();
	if (!Get_All_Moves (col, &mv_list, state)) return false;
	if (mv_list.Empty ()) { // player has not any move
		state->Set_Victory (true);
		if (col == BLACK) state->Set_Winner (WHITE); else state->Set_Winner (BLACK);
		state->Set_Game_Ended (true);
		val = true;
	}
	for (int i = 0; i != mv_list.Number_Of_Items (); i++) delete mv_list.Get_Value (i);
	val = false;
	return true;
}

bool CReferee::Back_Move ()
{
	CHistory* history = state->Get_History();
	if (history->Empty ()) return false;
	CMove* move = history->Get_Value (history->Number_Of_Items() - 1);
	if (!undo_list.Insert_Value(move)) return false;
	history->Remove_Item(history->Number_Of_Items() - 1);
	CHistory hist;
	if (!history->Get_Copy (&hist)) return false;
	Create_New_Game ();
	if (!Play_Move_List (&hist)) return false;
	for (int i = 0; i < hist.Number_Of_Items(); i++) delete hist.Get_Value(i);
	return true;
}

bool CReferee::Play_Move_List (m_CList<CMove*>* list)
{
	CMove* move;
	TColor c;
	for (int i = 0; i < list->Number_Of_Items (); i++) {
		move = list->Get_Value (i);
		c = state->Whos_On_Turn ();
		if (move->Number_Of_Positions() == 1) {
			if (!Remove_Figure(move->Get_Position(0), true)) return false;
		}
		else {	
			if (!Check_Move (move, c)) return false;
			if (!Execute_Move (move, NULL)) return false;
			if (c == WHITE) Set_Player_On_Turn (BLACK);
			else if (c == BLACK) Set_Player_On_Turn (WHITE);
		}
	}
	return true;
}

bool CReferee::Remove_Figure (CPosition& pos, bool really)
{
	if (last_move_mistake == 0) return false;
	bool is_in = false;
	for (int i = 0; i < last_move_mistake_positions.Number_Of_Items(); i++)
		if (last_move_mistake_positions.Get_Value(i) == pos) { is_in = true; break; }
	if (!is_in) return false;
	if (really) {
		last_move_mistake = 0;
		last_move_mistake_positions.Clear ();
		state->Remove_Figure(pos);
		CMove move;
		if (!move.Insert_Position(pos)) return false;
		move.Set_Player(Whos_On_Turn());
		if (!state->Set_Last_Move (&move)) return false; // save last move
	}
	return true;
}

bool CReferee::Get_Forced_Move(CMove* move)
{
	if (move_list.Empty()) return false;
	CMove* mv = move_list.Get_Value(0);
	if (move->Is_Jump())
		for (int i = 0; i < move_list.Number_Of_Items(); i++) {
			mv = move_list.Get_Value(i);
			int j = 0;
			for (; ((j < move->Number_Of_Positions()) && (j < mv->Number_Of_Positions())); j++)
				if (!(move->Get_Position(j) == mv->Get_Position(j))) break;
			if (j == move->Number_Of_Positions()) break;
		}
	*move = *mv;
	return true;
}

bool CReferee::End_Game (TColor c, bool vict, TColor loser, bool really)
{
	if (state->Whos_On_Turn() != c) return false;
	if (c != loser) return false;
	if (really) {
		state->Set_Victory (vict);
		if (loser == WHITE) state->Set_Winner (BLACK);
		else if (loser == BLACK) state->Set_Winner (WHITE);
		state->Set_Game_Ended (true);	
	}
	return true;
}

bool CReferee::Forward_Move ()
{
	if (undo_list.Empty()) return false;
	CMove* move = undo_list.Get_Value (undo_list.Number_Of_Items() - 1);
	if (move->Number_Of_Positions() == 1) {
		if (!Remove_Figure(move->Get_Position(0), true)) return false;
	}
	else {
		if (!Execute_Move(move, state)) return false;
		TColor c = state->Whos_On_Turn();
		if (c == WHITE) Set_Player_On_Turn (BLACK);
		else if (c == BLACK) Set_Player_On_Turn (WHITE);
	}
	delete move;
	undo_list.Remove_Item (undo_list.Number_Of_Items() - 1);
	return true;
}

void CReferee::Clear_Undo_List()
{
	for (int i = 0; i < undo_list.Number_Of_Items(); i++)
		delete undo_list.Get_Value(i);
	undo_list.Clear();
}

void CReferee::DataAsText(char* text)
{
	strcat(text, "Referee:##");
	strcat(text, "State: "); if (state != NULL) strcat(text, "OK#"); else strcat(text, "NULL#");
	strcat(text, "Must jump: ");
	if (must_jump) strcat(text, "TRUE#"); else strcat(text, "FALSE#");
	strcat(text, "No jump move count: ");
	char letter[2]; letter[1] = 0;
	*letter = nojump_move_count / 10 + '0'; strcat(text, letter);
	*letter = nojump_move_count % 10 + '0'; strcat(text, letter);
	strcat(text, "#");
	strcat(text, "Undo list: ");
	CMove* move;
	CPosition pos;
	for (int i = 0; i < undo_list.Number_Of_Items(); i++) {
		move = undo_list.Get_Value(i);
		if (move->Get_Player() == WHITE) strcat(text, "WHITE: "); else strcat(text, "BLACK: ");
		for (int k = 0; k < move->Number_Of_Positions(); k++) {
			pos = move->Get_Position(k);
			*letter = pos.Get_Vertical_Index() + '0'; strcat(text, letter);
			*letter = pos.Get_Horizontal_Index() + '0'; strcat(text, letter); strcat (text, " ");
		}
		strcat(text, "#");
	}
	strcat(text, "#");
	strcat(text, "Last move mistake: ");
	if (last_move_mistake == 2) strcat(text, "Incomplete multi-jump#");
	else if (last_move_mistake == 1) strcat(text, "Forgotten jump#"); else strcat(text, "No mistake#");
	strcat(text, "Last move mistake positions: ");
	for (int j = 0; j < last_move_mistake_positions.Number_Of_Items(); j++) {
			pos = last_move_mistake_positions.Get_Value(j);
			*letter = pos.Get_Vertical_Index() + '0'; strcat(text, letter);
			*letter = pos.Get_Horizontal_Index() + '0'; strcat(text, letter); strcat (text, " ");
	}
	strcat(text, "#");
}
