///////////////////////////
// WINDOWS SPECIFIC PART //

#include "stdafx.h"

///////////////////////////

#include "res.h"
#include "datastructs.h"
#include "stuff.h"
#include "state.h"

CState::CState () : white_man (MAN, WHITE), white_king (KING, WHITE), black_man (MAN, BLACK), black_king (KING, BLACK)
{
	game_ended = false;
	history = NULL;
	eval_value = 0;
	player_on_turn = NO_COLOR;
	victory = false;
	winner = NO_COLOR;
}

bool CState::Create (bool create_history)
{
	if (!board.Create ()) return false;
	if (create_history) {
		history = NULL;
		history = new CHistory ();
		if (history == NULL) return false;
	}
	return true;
}

CState::~CState ()
{
	if (history != NULL) {
		for (int i = 0; i < history->Number_Of_Items (); i++) delete history->Get_Value (i);
		delete history;
	}
}

void CState::Reset ()
{
	CFigure* fig = NULL;
	for (char i = 0; i < FIELD_COUNT_VERTICAL; i++)
		for (char j = 0; j < FIELD_COUNT_HORIZONTAL; j++) {
			fig = NULL;
			if (INITIAL_POSITIONS[i][j] == WHITE)
				fig = &white_man;
			if (INITIAL_POSITIONS[i][j] == BLACK)
				fig = &black_man;
			board.Put_Figure (CPosition (i, j), fig);
		}
	for (int k = 0; k < history->Number_Of_Items (); k++) delete history->Get_Value (k);
	history->Clear ();
}

CFigure* CState::Remove_Figure (CPosition& pos)
{
	CFigure* fig = board.Get_Figure (pos);
	board.Remove_Figure (pos);
	return fig;
}

void CState::Put_Figure (CPosition& pos, CFigure* fig)
{
	board.Put_Figure (pos, fig);
}

void CState::Put_Figure (CPosition& pos, TColor c, TFigure_Type t)
{
	if (c == WHITE)
		if (t == MAN) board.Put_Figure (pos, &white_man);
		else if (t == KING) board.Put_Figure (pos, &white_king);
	else if (c == BLACK)
		if (t == MAN) board.Put_Figure (pos, &black_man);
		else if (t == KING) board.Put_Figure (pos, &black_king);
}

void CState::Change_To_King (CPosition& pos)
{
	CFigure* fig = board.Get_Figure (pos);
	if (fig->Get_Color () == WHITE) board.Put_Figure (pos, &white_king);
	if (fig->Get_Color () == BLACK) board.Put_Figure (pos, &black_king);
}

const CState& CState::operator = (CState& state)
{
	if (this != &state) {
		board = *state.Get_Board ();
		last_move = *state.Get_Last_Move ();
	}
	return *this;
}

bool CState::Set_Last_Move (CMove* move)
{
	last_move = *move;
	if (history != NULL) {
		CMove* m = NULL;
		m = new CMove;
		if (m == NULL) return false;
		*m = *move;
		if (!history->Insert_Value (m)) return false;
	}
	return true;
}

void CState::Evaluate (int moves_w, int moves_b)
{
	const int MAN_WORTH = 50; // points for a man
	const int KING_WORTH = 150; // points for a king
	const int LOSS = -1000; //points for a loss/victory
	int count[PLAYER_COUNT], pos[PLAYER_COUNT];
	CFigure* fig;
	TColor col;
	TFigure_Type type;
	for (int k = 0; k < PLAYER_COUNT; k++) { count[k] = 0; pos[k] = 0; eval_value = 0; }
	for (char i = 0; i < FIELD_COUNT_VERTICAL; i++)
		for (char j = 0; j < FIELD_COUNT_HORIZONTAL; j++) {
			if ((fig = board.Get_Figure (CPosition (i, j))) == NULL) continue;
			col = fig->Get_Color (); type = fig->Get_Type ();
			if (type == KING) count[col] += KING_WORTH;
			else { 
				count[col] += MAN_WORTH;
				if (col == WHITE) pos[col] += i; 
				else pos[col] += ((FIELD_COUNT_VERTICAL - 1) - i);
			}
		}
	if ((!count[WHITE]) || (!moves_w)) count[WHITE] = LOSS;
	if ((!count[BLACK]) || (!moves_b)) count[BLACK] = LOSS;
	eval_value = count[WHITE] - count[BLACK] + 2 * (pos[WHITE] - pos[BLACK]) + moves_w - moves_b;
}

void CState::DataAsText(char* text)
{
	strcat (text, "State:##");
	strcat(text, "History: "); if (history != NULL) strcat(text, "OK#"); else strcat(text, "NULL#");
	strcat(text, "Evaluation value: "); if (eval_value < 0) strcat(text, "-");
	char letter[2]; letter[1] = 0;
	*letter = eval_value / 100 + '0'; strcat(text, letter);
	*letter = (eval_value % 100) / 10 + '0'; strcat(text, letter);
	*letter = eval_value % 10 + '0'; strcat(text, letter); strcat (text, "#");
	strcat(text, "Player on turn: ");
	if (player_on_turn == WHITE) strcat(text, "WHITE#");
	else if (player_on_turn == BLACK) strcat(text, "BLACK#"); else strcat(text, "NONE#");
	strcat(text, "Last move: ");
	if (last_move.Get_Player() == WHITE) strcat(text, "WHITE: "); else strcat(text, "BLACK: ");
	CPosition pos;
	for (int k = 0; k < last_move.Number_Of_Positions(); k++) {
			pos = last_move.Get_Position(k);
			*letter = pos.Get_Vertical_Index() + '0'; strcat(text, letter);
			*letter = pos.Get_Horizontal_Index() + '0'; strcat(text, letter); strcat (text, " ");
		}
	strcat(text, "#");
	strcat(text, "Game ended: ");
	if (game_ended) strcat(text, "TRUE#"); else strcat(text, "FALSE#");
	strcat(text, "Victory: ");
	if (victory) strcat(text, "TRUE#"); else strcat(text, "FALSE#");
	strcat(text, "Winner: ");
	if (winner == WHITE) strcat(text, "WHITE#"); 
	else if (winner == BLACK) strcat(text, "BLACK#"); else strcat(text, "NONE#");
}
